;################################################################################
;#										#
;# AX81 computer with ATmega644							#
;# copyright (c) 2010-2011 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License as		#
;# published by the Free Software Foundation; either version 3 of		#
;# the License, or (at your option) any later version.				#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.def	const_0 = r2
.def	const_1 = r3

.set	USE_I2C = 0

.include "config_mcu.asm"
.include "config_vsys.asm"
.include "config_fsys.asm"
.include "config_bootloader.asm"

.equ		libmio_ram	= 0x0100	;24 bytes libmio system RAM
.equ		z80emu_ram	= 0x0118	;32 bytes shadow register etc.
.equ		fileio_ram	= 0x0138	;40 bytes fileio ram
.equ		stack		= 0x01ff	;160 bytes stack
.equ		zx_ram		= 0x0200	;start of ZX memory

.equ		T_ERR		= 0x1c		;"T" error

.include	"libmio/definitions.asm"

.if MMAP == 100
	.set bl_buffer = 0x03
	.set STACKA = 0x03
.endif

.if MMAP == 175
	.set bl_buffer = 0x06
	.set STACKA = 0x07
.endif

.if MMAP == 200
	.set bl_buffer = 0x07
	.set STACKA = 0x07
.endif

.if MMAP == 375
	.set bl_buffer = 0x0f
	.set STACKA = 0x0f
.endif

.if MMAP == 1575
	.set bl_buffer = 0x3f
	.set STACKA = 0x3f
.endif

	.include	"libmio/timing_vga1.asm"
	.include	"libmio/timing_tv1.asm"

.if MCU < 400
	.include	"filesys/fconfig_28pin.asm"
.else
	.include	"filesys/fconfig_40pin.asm"
.endif

.include	"emulator/emu_defs.asm"
.include	"emulator/mem_macros.asm"
.include	"emulator/emacros.asm"
.if CSWAP == 0
	.include	"modules/csetdata_norm.inc"
.else
	.include	"modules/csetdata_swap.inc"
.endif


.CSEG

.org 0x0000
		jmp	start
.org 0x0004
		.db 0,1				;System/version
.org OC1Aaddr
		jmp	libmio_aisr		;video interrupt CHA
.org OC1Baddr
		jmp	libmio_bisr		;video interrupt CHB
.org OVF1addr
		jmp	libmio_oisr		;video interrupt OV

start:		cli				;disable video after jmp 0
		ldi	XL,0x00
		mov	const_0,XL
		ldi	XL,0x01
		mov	const_1,XL

		ldi	XL,HIGH(stack)
		out	SPH,XL
		ldi	XL,LOW(stack)
		out	SPL,XL
		out	SPCR,const_0		;disable SPI

.if MCU == 1284
		out	RAMPZ,const_1
.endif

		call	libmio_init			;activate video
;Dataflash needs a time to warmup
.if FSYS < 3
		clr	XL
		clr	XH
startw1:	sbiw	XL,1
		brne	startw1
.endif

		call	fsys_init			;
.if USE_BL == 1
		sbi	BODDR,BOPIN
		cbi	BIDDR,BIPIN
		sbi	BOPORT,BOPIN

		.if USE_CHP == 10
			.include	"modules/init_chpump_oc0a.asm"
		.endif
		.if USE_CHP == 11
			.include	"modules/init_chpump_oc0b.asm"
		.endif
		.if USE_CHP == 20
			.include	"modules/init_chpump_oc2a.asm"
		.endif
		.if USE_CHP == 21
			.include	"modules/init_chpump_oc2b.asm"
		.endif
.endif

		call	cset_intro			;intro screen

loop:		jmp	emu_start		;start emulator
		rjmp	loop

.include	"emulator/emulator.asm"
.include	"emulator/emulator_cb.asm"
.include	"emulator/emulator_dd.asm"
.include	"emulator/emulator_ddcb.asm"
.include	"emulator/emulator_fd.asm"
.include	"emulator/emulator_fdcb.asm"
.include	"emulator/emulator_ed.asm"
.include	"emulator/emulator_io.asm"


.if MCU == 328
	.if FSYS == 1
		.include	"filesys/df256.asm"
	.endif
	.if FSYS == 3
		.include	"filesys/sdv2.asm"
	.endif

	.if USE_BL == 1
		.include	"modules/bootloader.asm"
	.endif

.endif


.if MCU == 328
	.include	"libmio/library_28pin.asm"
.else
	.include	"libmio/library_40pin.asm"
.endif

.if USE_LCD == 1
	.include "libmio/init_lcd1.asm"
.endif

.if USE_TV > 0
	.include "libmio/init_tv1.asm"
.endif
.if USE_VGA > 0
	.include "libmio/init_vga1.asm"
.endif

.if USE_TV > 0
	.if USE_VGA == 0
		.include "libmio/vint_bisr.asm"
	.endif
.endif

.if USE_LCD == 1
	.include "libmio/vint_lcd1.asm"
.else
	libmio_oisr:	reti
.endif

.if USE_TV > 0
	.include "libmio/vint_tv1.asm"
.endif
.if USE_VGA > 0
	.include "libmio/vint_vga1.asm"
.endif

.include	"keyboard/kint.asm"



.org	0x2780
libmio_kstab:
.include	"keyboard/kstable.inc"

.org 0x2800
libmio_ktables:
.include	"keyboard/keytable_de.inc"
libmio_zktable:
.include	"keyboard/zxkeys.inc"

.org 0x2a00
zxmap:
.if MMAP == 100
	.include	"emulator/memmap_1K.asm"
.endif

.if MMAP == 175
	.include	"emulator/memmap_1K75.asm"
.endif

.if MMAP == 200
	.include	"emulator/memmap_2K.asm"
.endif

.if MMAP == 375
	.include	"emulator/memmap_3K75.asm"
.endif

.if MMAP == 1575
	.include	"emulator/memmap_15K75.asm"
.endif
.include	"emulator/flags.asm"

.org 0x2b80
libmio_ctable:
.include	"modules/csetcode.asm"



.org 0x2F80
.include	"emulator/eblock.inc"
.org 0x3000
.include	"emulator/zx81rom.inc"

.if MCU > 400
	.if FSYS == 1
		.include	"filesys/df256.asm"
	.endif
	.if FSYS == 3
		.include	"filesys/sdv2.asm"
	.endif

	.if USE_BL == 1
		.include	"modules/bootloader.asm"
	.endif
.endif
